Class: FeideeUtils::Transaction

Inherits:
Record
  • Object
show all
Includes:
Mixins::Type
Defined in:
lib/feidee_utils/transaction.rb

Defined Under Namespace

Classes: InconsistentAmountException, InconsistentBuyerAndSellerSetException, InconsistentCategoryException, ModifiedTransaction, TransferLackBuyerOrSellerException, TransferWithCategoryException, TransfersNotPaired

Constant Summary collapse

FieldMappings =
{
  raw_created_at:         "createdTime",
  raw_modified_at:        "modifiedTime",
  raw_trade_at:           "tradeTime",
  raw_type:               "type",
  memo:                   "memo",
  buyer_account_poid:     "buyerAccountPOID",
  buyer_category_poid:    "buyerCategoryPOID",
  seller_account_poid:    "sellerAccountPOID",
  seller_category_poid:   "sellerCategoryPOID",
  raw_buyer_deduction:    "buyerMoney",
  raw_seller_addition:    "sellerMoney",
  uuid:                   "relation",
}.freeze
IgnoredFields =
[
  "creatorTradingEntityPOID",
  "modifierTradingEntityPOID",
  "ffrom",                # The signature of the App writting this transaction.
  "photoName",            # To be added
  "photoNeedUpload",      # To be added
  "relationUnitPOID",     # WTF
  "clientID",             # WTF
  "FSourceKey",           # WTF
].freeze

Instance Attribute Summary

Attributes inherited from Record

#field, #field_type

Attributes included from Record::Namespaced::ClassMethods

#child_classes

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Mixins::Type

included, #type

Methods inherited from Record

#initialize

Methods included from Record::Accessors::ClassMethods

#define_accessors

Methods included from Record::Namespaced::ClassMethods

#collect_subclass, #generate_namespaced_record_classes

Methods included from Record::Persistent::ClassMethods

#all, #find, #find_by_id, #genereate_names

Methods included from Record::Accessors

#last_update_time, #poid

Constructor Details

This class inherits a constructor from FeideeUtils::Record

Class Method Details

.validate_integrity_globallyObject



67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
# File 'lib/feidee_utils/transaction.rb', line 67

def self.validate_integrity_globally
  uuids_map = all.inject({}) do |uuids, transaction|
    if transaction.is_transfer?
      uuid = transaction.uuid
      uuids[uuid] ||= [nil, nil]
      uuids[uuid][transaction.raw_type - 2] = transaction
    end
    uuids
  end

  uuids_map.each do |uuid, transfers|
    valid = true
    valid &&= transfers[0] != nil
    valid &&= transfers[1] != nil
    valid &&= transfers[0]. == transfers[1].
    valid &&= transfers[0]. == transfers[1].
    raise TransfersNotPaired.new([uuid] + transfers) unless valid
  end
end

Instance Method Details

#amountObject



155
156
157
158
# File 'lib/feidee_utils/transaction.rb', line 155

def amount
  # Buyer deduction is always equal to seller addition.
  (buyer_deduction + seller_addition) / 2
end

#buyer_deductionObject

Amount accessors



147
148
149
# File 'lib/feidee_utils/transaction.rb', line 147

def buyer_deduction
  sign_by_type(raw_buyer_deduction)
end

#category_poidObject



140
141
142
143
# File 'lib/feidee_utils/transaction.rb', line 140

def category_poid
  # At least one of those two must be 0.
  buyer_category_poid + seller_category_poid
end

#created_atObject



124
125
126
# File 'lib/feidee_utils/transaction.rb', line 124

def created_at
  timestamp_to_time(raw_created_at)
end

#has_category?Boolean

Returns:

  • (Boolean)


136
137
138
# File 'lib/feidee_utils/transaction.rb', line 136

def has_category?
  category_poid != 0
end

#is_initial_balance?Boolean

Returns:

  • (Boolean)


164
165
166
# File 'lib/feidee_utils/transaction.rb', line 164

def is_initial_balance?
  type == :positive_initial_balance or type == :negative_initial_balance
end

#is_transfer?Boolean

Returns:

  • (Boolean)


160
161
162
# File 'lib/feidee_utils/transaction.rb', line 160

def is_transfer?
  type == :transfer_buyer or type == :transfer_seller
end

#modified_atObject



128
129
130
# File 'lib/feidee_utils/transaction.rb', line 128

def modified_at
  timestamp_to_time(raw_modified_at)
end

#revised_account_poidObject



168
169
170
171
172
173
174
175
176
# File 'lib/feidee_utils/transaction.rb', line 168

def 
  if type == :transfer_buyer
    
  elsif type == :transfer_seller
    
  else
     + 
  end
end

#revised_amountObject



178
179
180
181
182
183
184
185
186
187
# File 'lib/feidee_utils/transaction.rb', line 178

def revised_amount
   = 
  if  == 
    -buyer_deduction
  elsif  == 
    seller_addition
  else
    raise "Unexpected revised account poid #{account_poid}."
  end
end

#seller_additionObject



151
152
153
# File 'lib/feidee_utils/transaction.rb', line 151

def seller_addition
  sign_by_type(raw_seller_addition)
end

#trade_atObject



132
133
134
# File 'lib/feidee_utils/transaction.rb', line 132

def trade_at
  timestamp_to_time(raw_trade_at)
end

#validate_integrityObject



23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
# File 'lib/feidee_utils/transaction.rb', line 23

def validate_integrity
  if is_transfer?
    unless  != 0 and  != 0
      raise TransferLackBuyerOrSellerException,
        "Both buyer and seller should be set in a transfer. " +
        "Buyer account POID: #{buyer_account_poid}. Seller account POID: #{seller_account_poid}.\n" +
        inspect
    end
    unless buyer_category_poid == 0 and seller_category_poid == 0
      raise TransferWithCategoryException,
        "Neither buyer or seller category should be set in a transfer. " +
        "Buyer category POID: #{buyer_category_poid}. Seller category POID: #{seller_category_poid}.\n" +
        inspect
    end
  else
    unless ( == 0) ^ ( == 0)
      raise InconsistentBuyerAndSellerSetException,
        "Exactly one of buyer and seller should be set in a non-transfer transaction. " +
        "Buyer account POID: #{buyer_account_poid}. Seller account POID: #{seller_account_poid}.\n" +
        inspect
    end

    # We could enforce that category is set to the matching party (buyer or seller) of account.
    # However the implementation could handle all situations, as long as only one of them is set.
    # Thus no extra check is done here.
    unless buyer_category_poid == 0 or seller_category_poid == 0
      raise InconsistentCategoryException,
        "Only one of buyer and seller category should be set in a non-transfer transaction. "
        "Buyer category POID: #{buyer_category_poid}. Seller category POID: #{seller_category_poid}.\n" +
        inspect
    end
  end

  unless buyer_deduction == seller_addition
    raise InconsistentAmountException,
      "Buyer and seller should have the same amount set. " +
      "Buyer deduction: #{buyer_deduction}, seller_addition: #{seller_addition}.\n" +
    inspect
  end
end